<?php

namespace RAP;

class LoginHandler {

    protected $locator;

    public function __construct(Locator $locator) {
        $this->locator = $locator;
    }

    /**
     * Returns redirect URL.
     */
    public function onIdentityDataReceived(Identity $identity): string {
        
        $this->locator->getSession()->setLoginIdentityType($identity->type);

        $userDao = $this->locator->getUserDAO();
        
        $user = $userDao->findUserByIdentity($identity->type, $identity->typedId);
        
        if ($user === null) {
            return $this->handleNewIdentity($identity);
        } else {
            $this->updateUser($user, $identity);
        }

        $session = $this->locator->getSession();
        if ($session->getOAuth2RequestData() === null && $session->getAction() === 'join' &&
                $session->getUser() !== null && $session->getUser()->id !== $user->id) {
            return $this->showConfirmJoin($user);
        }

        return $this->getAfterLoginRedirect($user);
    }

    protected function handleNewIdentity(Identity $identity): string {

        $session = $this->locator->getSession();

        if ($session->getUser() !== null && $session->getAction() === 'join') {
            $userToJoin = $this->getNewUser($identity);
            return $this->showConfirmJoin($userToJoin);
        } else {
            return $this->redirectToTOUCheck($identity);
        }
    }

    private function showConfirmJoin(User $userToJoin): string {
        $this->locator->getSession()->setUserToJoin($userToJoin);
        return $this->locator->getBasePath() . '/confirm-join';
    }

    /**
     * Stores the data into session and Redirect to Term of Use acceptance page.
     */
    private function redirectToTOUCheck(Identity $identity): string {

        // Create new user
        $user = $this->getNewUser($identity);

        $this->locator->getSession()->setUser($user);

        return $this->locator->getBasePath() . '/tou-check';
    }

    private function getNewUser(Identity $identity): User {
        $user = new User();
        $user->addIdentity($identity);
        return $user;
    }

    /**
     * Update user with fresh information received by IdP. Useful for keeping email address always updated.
     */
    private function updateUser(User $user, Identity $identity): void {
        $savedIdentity = $user->getIdentityByTypedId($identity->typedId);
        $savedIdentity->email = $identity->email;
        if($identity->name !== null) {
            $savedIdentity->name = $identity->name;
        }
        if($identity->surname !== null) {
            $savedIdentity->surname = $identity->surname;
        }
        if($identity->institution !== null) {
            $savedIdentity->institution = $identity->institution;
        }
        $this->locator->getUserDAO()->updateIdentity($savedIdentity);
    }

    public function getAfterLoginRedirect(User $user): string {

        $session = $this->locator->getSession();
        $this->locator->getAuditLogger()->info("LOGIN," . $session->getLoginIdentityType() . "," . $user->id);

        if ($session->getOAuth2RequestData() !== null) {
            $session->setUser($user);
            $redirectUrl = $this->locator->getOAuth2RequestHandler()->getRedirectResponseUrl();
            session_destroy();
            return $redirectUrl;
        }

        if ($session->getAction() !== null) {

            $action = $session->getAction();

            if ($action === 'join') {
                $user = $this->joinTo($user);
                $action = 'account';
                $session->setAction($action);
            }

            $session->setUser($user);

            if ($action === 'account' || $action === 'admin') {
                return $this->locator->getBasePath() . '/' . $action;
            }
        }

        throw new \Exception("Unable to find a proper redirect");
    }

    private function joinTo(User $userToJoin): User {

        $session = $this->locator->getSession();
        $user = $session->getUser();

        if ($user === null) {
            return $userToJoin;
        }

        if ($userToJoin->id === null) {
            // New identity, not yet associated with an user: simply add it to
            // previously logged in user.
            $identity = $userToJoin->identities[0];
            $user->addIdentity($identity);
            $this->locator->getUserHandler()->saveUser($user);
        } else if ($user->id !== $userToJoin->id) {
            $user = $this->locator->getUserHandler()->joinUsers($user, $userToJoin);
        }

        $session->setUserToJoin(null);

        return $user;
    }

}
